#!/bin/bash

###############################################################################################
#										begin pegaswitch									  #
###############################################################################################
#Checking is pegaswitch folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
pegaswitch_update_only=false
if [ -d "pegaswitch" ]
then
	if [ -f "pegaswitch/update_only" ]
	then
		pegaswitch_update_only=true 
	else
		echo "Trying to delete ~/pegaswitch for doing a clean git clone"
		sudo rm -r "pegaswitch"
		if [ -d "pegaswitch" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$pegaswitch_update_only" = true ]
then
	echo "Updating pegaswitch"
	pushd pegaswitch > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/reswitched/pegaswitch.git
	if ! [ -d "pegaswitch" ]
	then
		echo "Error: Could not clone the pegaswitch repository."
		echo "Exiting."
		exit 1 
	fi
	touch "pegaswitch/update_only"
fi

pushd pegaswitch > /dev/null 2>&1
# Install the pegaswitch dependencies
npm i
# Create nros directory if needed
if ! [ -d "nros" ]
then
	mkdir nros
fi
popd > /dev/null 2>&1 # pegaswitch
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end pegaswitch										  #
###############################################################################################

###############################################################################################
#										begin libtransistor									  #
###############################################################################################
#Checking if pip3 exists, and if not install it.
if ! [ -x "$(command -v pip3)" ]
then
	echo "Error: python3-pip is not installed." 
	echo "Will try to install it now." 
	sudo apt-get install python3-pip
	if ! [ -x "$(command -v pip3)" ]
	then
		echo "Error: python3-pip could not be installed." 
		echo "Exiting."
		exit 1 		
	fi
fi

#Checking is libtransistor folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
libtransistor_update_only=false
if [ -d "libtransistor" ]
then
	if [ -f "libtransistor/update_only" ]
	then
		libtransistor_update_only=true 
	else
		echo "Trying to delete ~/libtransistor for doing a clean git clone"
		sudo rm -r "libtransistor"
		if [ -d "libtransistor" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$libtransistor_update_only" = true ]
then
	echo "Updating libtransistor"
	pushd libtransistor > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/reswitched/libtransistor.git
	if ! [ -d "libtransistor" ]
	then
		echo "Error: Could not clone the libtransistor repository."
		echo "Exiting."
		exit 1 
	fi
	touch "libtransistor/update_only"
fi

pushd libtransistor > /dev/null 2>&1
# Install the python dependencies. On ArchLinux
pip install -r requirements.txt
# On everything else
pip3 install -r requirements.txt
#Adding the flags sugested on the below link so some cores that use CXX can be compiled with retroarch
#https://gbatemp.net/threads/how-to-build-vbanext-only-for-people-with-ra-and-pega.493795/
if grep -Fq "CXXFLAGS" libtransistor.mk
then
    # code if found
	echo "CXXFLAGS already declared on libtransistor.mk"
else
    # code if not found
	echo "CXXFLAGS += \$(CC_FLAGS)" >> libtransistor.mk
fi
# Compile libtransistor
echo "Compiling libtransistor"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0
# Compile ace_loader
pushd projects/ace_loader > /dev/null 2>&1
echo "Compiling ace_loader"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 
cp ace.nro ../../../pegaswitch/nros/ace.nro
popd > /dev/null 2>&1 # projects/ace_loader
popd > /dev/null 2>&1 # libtransistor
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end libtransistor									  #
###############################################################################################

###############################################################################################
#										begin libretro-fceumm								  #
###############################################################################################
#Checking is libretro-fceumm folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
libretro_fceumm_update_only=false
if [ -d "libretro-fceumm" ]
then
	if [ -f "libretro-fceumm/update_only" ]
	then
		libretro_fceumm_update_only=true 
	else
		echo "Trying to delete ~/libretro-fceumm for doing a clean git clone"
		sudo rm -r "libretro-fceumm"
		if [ -d "libretro-fceumm" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$libretro_fceumm_update_only" = true ]
then
	echo "Updating libretro-fceumm"
	pushd libretro-fceumm > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/libretro-fceumm.git
	if ! [ -d "libretro-fceumm" ]
	then
		echo "Error: Could not clone the libretro-fceumm repository."
		echo "Exiting."
		exit 1 
	fi
	touch "libretro-fceumm/update_only"
fi

pushd libretro-fceumm > /dev/null 2>&1
echo "Compiling libretro-fceumm"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # libretro-fceumm
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end libretro-fceumm					 				  #
###############################################################################################

###############################################################################################
#										begin libtransistor-snes9x2010						  #
###############################################################################################
#Checking is libtransistor-snes9x2010 folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
libtransistor_snes9x2010_update_only=false
if [ -d "libtransistor-snes9x2010" ]
then
	if [ -f "libtransistor-snes9x2010/update_only" ]
	then
		libtransistor_snes9x2010_update_only=true 
	else
		echo "Trying to delete ~/libtransistor-snes9x2010 for doing a clean git clone"
		sudo rm -r "libtransistor-snes9x2010"
		if [ -d "libtransistor-snes9x2010" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$libtransistor_snes9x2010_update_only" = true ]
then
	echo "Updating libtransistor-snes9x2010"
	pushd libtransistor-snes9x2010 > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/reswitched/libtransistor-snes9x2010.git
	if ! [ -d "libtransistor-snes9x2010" ]
	then
		echo "Error: Could not clone the libtransistor-snes9x2010 repository."
		echo "Exiting."
		exit 1 
	fi
	touch "libtransistor-snes9x2010/update_only"
fi

pushd libtransistor-snes9x2010 > /dev/null 2>&1
echo "Compiling libtransistor-snes9x2010"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # libtransistor-snes9x2010
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end libtransistor-snes9x2010		 				  #
###############################################################################################

###############################################################################################
#										begin vba-next										  #
###############################################################################################
#Checking is vba-next folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
vba_next_update_only=false
if [ -d "vba-next" ]
then
	if [ -f "vba-next/update_only" ]
	then
		vba_next_update_only=true 
	else
		echo "Trying to delete ~/vba-next for doing a clean git clone"
		sudo rm -r "vba-next"
		if [ -d "vba-next" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$vba_next_update_only" = true ]
then
	echo "Updating vba-next"
	pushd vba-next > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/vba-next.git
	if ! [ -d "vba-next" ]
	then
		echo "Error: Could not clone the vba-next repository."
		echo "Exiting."
		exit 1 
	fi
	touch "vba-next/update_only"
fi

pushd vba-next > /dev/null 2>&1
echo "Compiling vba-next"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # vba-next
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end vba-next						 				  #
###############################################################################################

###############################################################################################
#										begin virtualjaguar-libretro						  #
###############################################################################################
#Checking is virtualjaguar-libretro folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
virtualjaguar_libretro_update_only=false
if [ -d "virtualjaguar-libretro" ]
then
	if [ -f "virtualjaguar-libretro/update_only" ]
	then
		virtualjaguar_libretro_update_only=true 
	else
		echo "Trying to delete ~/virtualjaguar-libretro for doing a clean git clone"
		sudo rm -r "virtualjaguar-libretro"
		if [ -d "virtualjaguar-libretro" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$virtualjaguar_libretro_update_only" = true ]
then
	echo "Updating virtualjaguar-libretro"
	pushd virtualjaguar-libretro > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/virtualjaguar-libretro.git
	if ! [ -d "virtualjaguar-libretro" ]
	then
		echo "Error: Could not clone the virtualjaguar-libretro repository."
		echo "Exiting."
		exit 1 
	fi
	touch "virtualjaguar-libretro/update_only"
fi

pushd virtualjaguar-libretro > /dev/null 2>&1
echo "Compiling virtualjaguar-libretro"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # virtualjaguar-libretro
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end virtualjaguar-libretro						 	  #
###############################################################################################


###############################################################################################
#										begin picodrive										  #
###############################################################################################
#Checking is picodrive folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
picodrive_update_only=false
if [ -d "picodrive" ]
then
	if [ -f "picodrive/update_only" ]
	then
		picodrive_update_only=true 
	else
		echo "Trying to delete ~/picodrive for doing a clean git clone"
		sudo rm -r "picodrive"
		if [ -d "picodrive" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$picodrive_update_only" = true ]
then
	echo "Updating picodrive"
	pushd picodrive > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/picodrive.git
	if ! [ -d "picodrive" ]
	then
		echo "Error: Could not clone the picodrive repository."
		echo "Exiting."
		exit 1 
	fi
	touch "picodrive/update_only"
fi

pushd picodrive > /dev/null 2>&1
echo "Compiling picodrive"
make clean -f Makefile.libretro LLVM_POSTFIX=-5.0
make -f Makefile.libretro LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # picodrive
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end picodrive						 				  #
###############################################################################################


###############################################################################################
#										begin libretro-handy								  #
###############################################################################################
#Checking is libretro-handy folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
libretro_handy_update_only=false
if [ -d "libretro-handy" ]
then
	if [ -f "libretro-handy/update_only" ]
	then
		libretro_handy_update_only=true 
	else
		echo "Trying to delete ~/libretro-handy for doing a clean git clone"
		sudo rm -r "libretro-handy"
		if [ -d "libretro-handy" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$libretro_handy_update_only" = true ]
then
	echo "Updating libretro-handy"
	pushd libretro-handy > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/libretro-handy.git
	if ! [ -d "libretro-handy" ]
	then
		echo "Error: Could not clone the libretro-handy repository."
		echo "Exiting."
		exit 1 
	fi
	touch "libretro-handy/update_only"
fi

pushd libretro-handy > /dev/null 2>&1
echo "Compiling libretro-handy"
make clean LLVM_POSTFIX=-5.0
make -lstdc++ LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # libretro-handy
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end libretro-handy						 			  #
###############################################################################################


###############################################################################################
#										begin yabause										  #
###############################################################################################
#Checking is yabause folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
yabause_update_only=false
if [ -d "yabause" ]
then
	if [ -f "yabause/update_only" ]
	then
		yabause_update_only=true 
	else
		echo "Trying to delete ~/yabause for doing a clean git clone"
		sudo rm -r "yabause"
		if [ -d "yabause" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$yabause_update_only" = true ]
then
	echo "Updating yabause"
	pushd yabause > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/yabause.git
	if ! [ -d "yabause" ]
	then
		echo "Error: Could not clone the yabause repository."
		echo "Exiting."
		exit 1 
	fi
	touch "yabause/update_only"
fi

pushd yabause/libretro > /dev/null 2>&1
echo "Compiling yabause"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # yabause/libretro
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end yabause						 				 	  #
###############################################################################################


###############################################################################################
#										begin fbalpha										  #
###############################################################################################
#Checking is fbalpha folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
fbalpha_update_only=false
if [ -d "fbalpha" ]
then
	if [ -f "fbalpha/update_only" ]
	then
		fbalpha_update_only=true 
	else
		echo "Trying to delete ~/fbalpha for doing a clean git clone"
		sudo rm -r "fbalpha"
		if [ -d "fbalpha" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$fbalpha_update_only" = true ]
then
	echo "Updating fbalpha"
	pushd fbalpha > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/fbalpha.git
	if ! [ -d "fbalpha" ]
	then
		echo "Error: Could not clone the fbalpha repository."
		echo "Exiting."
		exit 1 
	fi
	touch "fbalpha/update_only"
fi

pushd fbalpha > /dev/null 2>&1
echo "Compiling fbalpha"
#make clean -f makefile.libretro LLVM_POSTFIX=-5.0
#make -f makefile.libretro LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # fbalpha
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end fbalpha						 				 	  #
###############################################################################################

###############################################################################################
#										begin Genesis-Plus-GX								  #
###############################################################################################
#Checking is Genesis-Plus-GX folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
Genesis_Plus_GX_update_only=false
if [ -d "Genesis-Plus-GX" ]
then
	if [ -f "Genesis-Plus-GX/update_only" ]
	then
		Genesis_Plus_GX_update_only=true 
	else
		echo "Trying to delete ~/Genesis-Plus-GX for doing a clean git clone"
		sudo rm -r "Genesis-Plus-GX"
		if [ -d "Genesis-Plus-GX" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$Genesis_Plus_GX_update_only" = true ]
then
	echo "Updating Genesis-Plus-GX"
	pushd Genesis-Plus-GX > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/Genesis-Plus-GX.git
	if ! [ -d "Genesis-Plus-GX" ]
	then
		echo "Error: Could not clone the Genesis-Plus-GX repository."
		echo "Exiting."
		exit 1 
	fi
	touch "Genesis-Plus-GX/update_only"
fi

pushd Genesis-Plus-GX > /dev/null 2>&1
echo "Compiling Genesis-Plus-GX"
make clean -f Makefile.libretro LLVM_POSTFIX=-5.0
make -f Makefile.libretro LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # Genesis-Plus-GX
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end Genesis-Plus-GX						 		 	  #
###############################################################################################

###############################################################################################
#										begin Gearboy										  #
###############################################################################################
#Checking is Gearboy folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
Gearboy_update_only=false
if [ -d "Gearboy" ]
then
	if [ -f "Gearboy/update_only" ]
	then
		Gearboy_update_only=true 
	else
		echo "Trying to delete ~/Gearboy for doing a clean git clone"
		sudo rm -r "Gearboy"
		if [ -d "Gearboy" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$Gearboy_update_only" = true ]
then
	echo "Updating Gearboy"
	pushd Gearboy > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/libretro/Gearboy.git
	if ! [ -d "Gearboy" ]
	then
		echo "Error: Could not clone the Gearboy repository."
		echo "Exiting."
		exit 1 
	fi
	touch "Gearboy/update_only"
fi

pushd Gearboy/platforms/libretro/ > /dev/null 2>&1
echo "Compiling Gearboy"
make clean LLVM_POSTFIX=-5.0
make LLVM_POSTFIX=-5.0 platform=switch LIBTRANSISTOR_HOME=/home/$USER/libtransistor/
popd > /dev/null 2>&1 # Gearboy/platforms/libretro/
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end Gearboy						 				 	  #
###############################################################################################

###############################################################################################
#										begin RetroArch										  #
###############################################################################################
#Checking is RetroArch folder exists
#if the file for update only git exists we will only update
#else we will remove the folder to do a clean clone of the repository
pushd ~/ > /dev/null 2>&1
RetroArch_update_only=false
if [ -d "RetroArch" ]
then
	if [ -f "RetroArch/update_only" ]
	then
		RetroArch_update_only=true 
	else
		echo "Trying to delete ~/RetroArch for doing a clean git clone"
		sudo rm -r "RetroArch"
		if [ -d "RetroArch" ]
		then
			echo "Error: path failed to be deleted."
			echo "Exiting."
			exit 1 
		fi
	fi
fi
#Now that we know what we need to do lets do it
if [ "$RetroArch_update_only" = true ]
then
	echo "Updating RetroArch"
	pushd RetroArch > /dev/null 2>&1
	git fetch --all
	git reset --hard origin/master
	touch "update_only"
	popd > /dev/null 2>&1
else
	git clone --recursive https://github.com/reswitched/RetroArch.git
	if ! [ -d "RetroArch" ]
	then
		echo "Error: Could not clone the RetroArch repository."
		echo "Exiting."
		exit 1 
	fi
	touch "RetroArch/update_only"
fi

pushd RetroArch > /dev/null 2>&1
# Create fs directory if needed
if ! [ -d "fs" ]
then
	mkdir fs
fi
popd > /dev/null 2>&1 # RetroArch
popd > /dev/null 2>&1 # ~/
###############################################################################################
#										end RetroArch						 				  #
###############################################################################################

echo "Done. Copy your roms to RetroArch/fs and run the buildretro script"